@@ -606,50 +606,50 @@ struct permonst mons[] = {
 	{ "grey dragon", S_DRAGON, 10, 9, -1, 20, 4, (G_GENO | 1),
 	  { { AT_BREA, AD_MAGM, 4, 6 }, { AT_BITE, AD_PHYS, 3, 8 },
 	    { AT_CLAW, AD_PHYS, 1, 4 }, { AT_CLAW, AD_PHYS, 1, 4 }, NO_ATTK },
-	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_NOHANDS | M1_EGGS |
-	  M1_GREEDY | M1_JEWELS | M1_MAGIC,
+	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_SEE_INVIS | M1_NOHANDS |
+	  M1_EGGS | M1_GREEDY | M1_JEWELS | M1_MAGIC,
 	  M2_HOSTILE | M2_STRONG | M2_NASTY | M2_CARNIVORE | M2_THICK_HIDE },
 	{ "red dragon", S_DRAGON, 10, 9, -1, 20, -4, (G_GENO | 1),
 	  { { AT_BREA, AD_FIRE, 6, 6 }, { AT_BITE, AD_PHYS, 3, 8 },
 	    { AT_CLAW, AD_PHYS, 1, 4 }, { AT_CLAW, AD_PHYS, 1, 4 }, NO_ATTK },
-	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_NOHANDS | M1_EGGS |
-	  M1_FIRE_RES | M1_GREEDY | M1_JEWELS | M1_MAGIC,
+	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_SEE_INVIS | M1_NOHANDS |
+	  M1_EGGS | M1_FIRE_RES | M1_GREEDY | M1_JEWELS | M1_MAGIC,
 	  M2_HOSTILE | M2_STRONG | M2_NASTY | M2_CARNIVORE | M2_THICK_HIDE },
 	{ "orange dragon", S_DRAGON, 10, 9, -1, 20, 5, (G_GENO | 1),
 	  { { AT_BREA, AD_SLEE, 4, 25 }, { AT_BITE, AD_PHYS, 3, 8 },
 	    { AT_CLAW, AD_PHYS, 1, 4 }, { AT_CLAW, AD_PHYS, 1, 4 }, NO_ATTK },
-	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_NOHANDS | M1_EGGS |
-	  M1_SLEE_RES | M1_GREEDY | M1_JEWELS | M1_MAGIC,
+	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_SEE_INVIS | M1_NOHANDS |
+	  M1_EGGS | M1_SLEE_RES | M1_GREEDY | M1_JEWELS | M1_MAGIC,
 	  M2_HOSTILE | M2_STRONG | M2_NASTY | M2_CARNIVORE | M2_THICK_HIDE },
 	{ "white dragon", S_DRAGON, 10, 9, -1, 20, -5, (G_GENO | 1),
 	  { { AT_BREA, AD_COLD, 4, 6 }, { AT_BITE, AD_PHYS, 3, 8 },
 	    { AT_CLAW, AD_PHYS, 1, 4 }, { AT_CLAW, AD_PHYS, 1, 4 }, NO_ATTK },
-	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_NOHANDS | M1_EGGS |
-	  M1_COLD_RES | M1_GREEDY | M1_JEWELS | M1_MAGIC,
+	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_SEE_INVIS | M1_NOHANDS |
+	  M1_EGGS | M1_COLD_RES | M1_GREEDY | M1_JEWELS | M1_MAGIC,
 	  M2_HOSTILE | M2_STRONG | M2_NASTY | M2_CARNIVORE | M2_THICK_HIDE },
 	{ "black dragon", S_DRAGON, 10, 9, -1, 20, -6, (G_GENO | 1),
 	  { { AT_BREA, AD_DISN, 4, 10 }, { AT_BITE, AD_PHYS, 3, 8 },
 	    { AT_CLAW, AD_PHYS, 1, 4 }, { AT_CLAW, AD_PHYS, 1, 4 }, NO_ATTK },
-	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_NOHANDS | M1_EGGS |
-	  M1_GREEDY | M1_JEWELS | M1_MAGIC,
+	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_SEE_INVIS | M1_NOHANDS |
+	  M1_EGGS | M1_GREEDY | M1_JEWELS | M1_MAGIC,
 	  M2_HOSTILE | M2_STRONG | M2_NASTY | M2_CARNIVORE | M2_THICK_HIDE },
 	{ "blue dragon", S_DRAGON, 10, 9, -1, 20, -7, (G_GENO | 1),
 	  { { AT_BREA, AD_ELEC, 4, 6 }, { AT_BITE, AD_PHYS, 3, 8 },
 	    { AT_CLAW, AD_PHYS, 1, 4 }, { AT_CLAW, AD_PHYS, 1, 4 }, NO_ATTK },
-	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_NOHANDS | M1_EGGS |
-	  M1_ELEC_RES | M1_GREEDY | M1_JEWELS | M1_MAGIC,
+	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_SEE_INVIS | M1_NOHANDS |
+	  M1_EGGS | M1_ELEC_RES | M1_GREEDY | M1_JEWELS | M1_MAGIC,
 	  M2_HOSTILE | M2_STRONG | M2_NASTY | M2_CARNIVORE | M2_THICK_HIDE },
 	{ "green dragon", S_DRAGON, 10, 9, -1, 20, 6, (G_GENO | 1),
 	  { { AT_BREA, AD_DRST, 4, 6 }, { AT_BITE, AD_PHYS, 3, 8 },
 	    { AT_CLAW, AD_PHYS, 1, 4 }, { AT_CLAW, AD_PHYS, 1, 4 }, NO_ATTK },
-	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_NOHANDS | M1_EGGS |
-	  M1_POIS | M1_POIS_RES | M1_GREEDY | M1_JEWELS | M1_MAGIC,
+	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_SEE_INVIS | M1_NOHANDS |
+	  M1_EGGS | M1_POIS | M1_POIS_RES | M1_GREEDY | M1_JEWELS | M1_MAGIC,
 	  M2_HOSTILE | M2_STRONG | M2_NASTY | M2_CARNIVORE | M2_THICK_HIDE },
 	{ "yellow dragon", S_DRAGON, 10, 9, -1, 20, 7, (G_GENO | 1),
 	  { { AT_BREA, AD_ACID, 4, 6 }, { AT_BITE, AD_PHYS, 3, 8 },
 	    { AT_CLAW, AD_PHYS, 1, 4 }, { AT_CLAW, AD_PHYS, 1, 4 }, NO_ATTK },
-	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_NOHANDS | M1_EGGS |
-	  M1_POIS | M1_STON_RES | M1_GREEDY | M1_JEWELS | M1_MAGIC,
+	  150, 1500, 0, MS_ROAR, M1_BIG | M1_FLY | M1_SEE_INVIS | M1_NOHANDS |
+	  M1_EGGS | M1_POIS | M1_STON_RES | M1_GREEDY | M1_JEWELS | M1_MAGIC,
 	  M2_HOSTILE | M2_STRONG | M2_NASTY | M2_CARNIVORE | M2_THICK_HIDE },
 /*	Elementals	*/
 	{ "air elemental", S_ELEMENTAL, 8, 36, 2, 30, 0, (G_NOCORPSE | 1),
@@ -1006,7 +1006,7 @@ struct permonst _mons2[] = {
 	  { { AT_WEAP, AD_DRLI, 1, 4 }, { AT_BREA, AD_SLEE, 0, 0 },
 	    NO_ATTK, NO_ATTK, NO_ATTK }, 30, 0, 0, MS_SILENT,
 	  M1_HUMANOID | M1_UNDEAD | M1_POIS_RES | M1_STALK | M1_COLLECT, 
-	  M2_STRONG },
+	  M2_STRONG | M2_HOSTILE },
 #endif
 /*	Xorn	*/
 	{ "xorn", S_XORN, 8, 9,-2, 20, 0, (G_GENO | 1),
@@ -1215,8 +1215,8 @@ struct permonst _mons2[] = {
 	  (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ),
 	  { { AT_WEAP, AD_PHYS, 1, 8 }, { AT_TUCH, AD_PLYS, 1, 8 },
 	    NO_ATTK, NO_ATTK, NO_ATTK }, 45, 400, PL_NSIZ, MS_FERRY,
-	  M1_NOPOLY | M1_HUMANOID | M1_POIS_RES | M1_FIRE_RES | M1_COLLECT,
-	  M2_HUMAN | M2_PEACEFUL },
+	  M1_NOPOLY | M1_SEE_INVIS | M1_HUMANOID | M1_POIS_RES | M1_FIRE_RES |
+	  M1_COLLECT, M2_HUMAN | M2_PEACEFUL },
 #endif
 /*	Ghost		*/
 	{ "ghost", S_GHOST, 10, 3, -5, 50, -5, (G_NOCORPSE | G_NOGEN),
@@ -1275,8 +1275,8 @@ struct permonst _mons2[] = {
 	  { { AT_WEAP, AD_PHYS, 2, 4 }, { AT_WEAP, AD_PHYS, 2, 4 },
 	    { AT_WEAP, AD_PHYS, 2, 4 }, { AT_WEAP, AD_PHYS, 2, 4 },
 	    { AT_WEAP, AD_PHYS, 2, 4 }, }, 45, 450, 0, MS_SILENT, M1_HUMANOID |
-	  M1_POIS | M1_FIRE_RES | M1_STALK | M1_COLLECT | M1_FEM,
-	  M2_DEMON | M2_HOSTILE | M2_NASTY },
+	  M1_SEE_INVIS | M1_POIS | M1_FIRE_RES | M1_STALK | M1_COLLECT |
+	  M1_FEM, M2_DEMON | M2_HOSTILE | M2_NASTY },
 	{ "barbed devil", S_DEMON, 8, 12, 0, 35, 8,
 	  (G_HELL | G_NOCORPSE | G_SGROUP | 2),
 	  { { AT_CLAW, AD_PHYS, 2, 4 }, { AT_CLAW, AD_PHYS, 2, 4 },
@@ -1310,63 +1310,65 @@ struct permonst _mons2[] = {
 	{ "ice devil", S_DEMON, 11, 6, -4, 55, -12, (G_HELL | G_NOCORPSE | 2),
 	  { { AT_CLAW, AD_PHYS, 1, 4 }, { AT_CLAW, AD_PHYS, 1, 4 },
 	    { AT_BITE, AD_PHYS, 2, 4 }, { AT_STNG, AD_COLD, 3, 4 }, NO_ATTK },
-	  45, 450, 0, MS_SILENT, M1_POIS | M1_FIRE_RES | M1_COLD_RES | M1_STALK,
-	  M2_DEMON | M2_HOSTILE | M2_NASTY },
+	  45, 450, 0, MS_SILENT, M1_SEE_INVIS | M1_POIS | M1_FIRE_RES |
+	  M1_COLD_RES | M1_STALK, M2_DEMON | M2_HOSTILE | M2_NASTY },
 	{ "pit fiend", S_DEMON, 13, 6, -3, 65, -13, (G_HELL | G_NOCORPSE | 2),
 	  { { AT_WEAP, AD_PHYS, 4, 2 }, { AT_WEAP, AD_PHYS, 4, 2 },
 	    { AT_HUGS, AD_PHYS, 2, 4 }, NO_ATTK, NO_ATTK }, 45, 450, 0,
-	  MS_GROWL, M1_POIS | M1_FIRE_RES | M1_STALK | M1_COLLECT,
-	  M2_DEMON | M2_HOSTILE | M2_NASTY },
+	  MS_GROWL, M1_SEE_INVIS | M1_POIS | M1_FIRE_RES | M1_STALK |
+	  M1_COLLECT, M2_DEMON | M2_HOSTILE | M2_NASTY },
 	{ "balrog", S_DEMON, 16, 5, -2, 75, -14, (G_HELL | G_NOCORPSE | 1),
 	  { { AT_WEAP, AD_PHYS, 8, 4 }, { AT_WEAP, AD_PHYS, 4, 6 },
-	    NO_ATTK, NO_ATTK, NO_ATTK },
-	  45, 450, 0, MS_SILENT, M1_FLY | M1_BIG | M1_COLLECT | M1_STALK |
-	  M1_POIS | M1_FIRE_RES, M2_DEMON | M2_HOSTILE | M2_STRONG | M2_NASTY },
+	    NO_ATTK, NO_ATTK, NO_ATTK }, 45, 450, 0, MS_SILENT,
+	  M1_FLY | M1_BIG | M1_SEE_INVIS | M1_COLLECT | M1_STALK | M1_POIS |
+	  M1_FIRE_RES, M2_DEMON | M2_HOSTILE | M2_STRONG | M2_NASTY },
 	/* Named demon lords & princes plus Arch-Devils */
 	{ "Juiblex", S_DEMON, 50, 3, -7, 65, -15,
 	  (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ),
 	  { { AT_ENGL, AD_DISE, 4, 10 }, { AT_SPIT, AD_ACID, 3, 6 },
 	    NO_ATTK, NO_ATTK, NO_ATTK }, 50, 500, PL_NSIZ, MS_GURGLE,
-	  M1_NOPOLY | M1_FLY | M1_NOHANDS | M1_POIS | M1_FIRE_RES | M1_STALK |
-	  M1_LORD, M2_DEMON | M2_HOSTILE | M2_PNAME | M2_NASTY | M2_AMORPHOUS },
+	  M1_NOPOLY | M1_FLY | M1_SEE_INVIS | M1_NOHANDS | M1_POIS |
+	  M1_FIRE_RES | M1_STALK | M1_LORD,
+	  M2_DEMON | M2_HOSTILE | M2_PNAME | M2_NASTY | M2_AMORPHOUS },
 	{ "Yeenoghu", S_DEMON, 56, 18, -5, 80, -15,
 	  (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ),
 	  { { AT_WEAP, AD_PHYS, 3, 6 }, { AT_WEAP, AD_CONF, 2, 8 },
 	    { AT_WEAP, AD_PLYS, 1, 6 }, { AT_MAGC, AD_MAGM, 2, 6 }, NO_ATTK },
-	  50, 500, PL_NSIZ, MS_ORC, M1_NOPOLY | M1_FLY | M1_POIS |
-	  M1_FIRE_RES | M1_STALK | M1_COLLECT | M1_LORD,
+	  50, 500, PL_NSIZ, MS_ORC, M1_NOPOLY | M1_FLY | M1_SEE_INVIS |
+	  M1_POIS | M1_FIRE_RES | M1_STALK | M1_COLLECT | M1_LORD,
 	  M2_DEMON | M2_HOSTILE | M2_PNAME | M2_NASTY },
 	{ "Orcus", S_DEMON, 66, 9, -6, 85, -20,
 	  (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ),
 	  { { AT_MAGC, AD_SPEL, 8, 6 }, { AT_WEAP, AD_PHYS, 3, 6 },
 	    { AT_CLAW, AD_PHYS, 3, 4 }, { AT_CLAW, AD_PHYS, 3, 4 },
 	    { AT_STNG, AD_DRST, 2, 4 }, }, 50, 500, PL_NSIZ, MS_ORC,
-	  M1_NOPOLY | M1_FLY | M1_POIS | M1_FIRE_RES | M1_STALK |
-	  M1_COLLECT | M1_PRINCE, M2_DEMON | M2_HOSTILE | M2_PNAME | M2_NASTY },
+	  M1_NOPOLY | M1_FLY | M1_SEE_INVIS | M1_POIS | M1_FIRE_RES |
+	  M1_STALK | M1_COLLECT | M1_PRINCE,
+	  M2_DEMON | M2_HOSTILE | M2_PNAME | M2_NASTY },
 	{ "Geryon", S_DEMON, 72, 3, -3, 75, 15,
 	  (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ),
 	  { { AT_CLAW, AD_PHYS, 3, 6 }, { AT_CLAW, AD_PHYS, 3, 6 },
 	    { AT_STNG, AD_DRST, 2, 4 }, NO_ATTK, NO_ATTK }, 50, 500, PL_NSIZ,
-	  MS_JEER, M1_NOPOLY | M1_FLY | M1_POIS | M1_FIRE_RES |
+	  MS_JEER, M1_NOPOLY | M1_FLY | M1_SEE_INVIS | M1_POIS | M1_FIRE_RES |
 	  M1_STALK | M1_PRINCE, M2_DEMON | M2_HOSTILE | M2_PNAME | M2_NASTY },
 	{ "Dispater", S_DEMON, 78, 15, -2, 80, 15,
 	  (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ),
 	  { { AT_WEAP, AD_PHYS, 4, 6 }, { AT_MAGC, AD_SPEL, 6, 6 },
 	    NO_ATTK, NO_ATTK, NO_ATTK }, 50, 500, PL_NSIZ, MS_JEER,
-	  M1_NOPOLY | M1_FLY | M1_HUMANOID | M1_POIS |
+	  M1_NOPOLY | M1_FLY | M1_SEE_INVIS | M1_HUMANOID | M1_POIS |
 	  M1_FIRE_RES | M1_STALK | M1_COLLECT | M1_PRINCE,
 	  M2_DEMON | M2_HOSTILE | M2_PNAME | M2_NASTY },
-	{ "Baalzebul", S_DEMON, 89, 9, -5, 85, 20,
+	{ "Baalzebub", S_DEMON, 89, 9, -5, 85, 20,
 	  (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ),
 	  { { AT_BITE, AD_DRST, 2, 6 }, { AT_GAZE, AD_STUN, 2, 6 },
 	    NO_ATTK, NO_ATTK, NO_ATTK }, 50, 500, PL_NSIZ, MS_JEER,
-	  M1_NOPOLY | M1_FLY | M1_POIS | M1_FIRE_RES | M1_STALK |
-	  M1_PRINCE, M2_DEMON | M2_HOSTILE | M2_PNAME | M2_NASTY },
+	  M1_NOPOLY | M1_FLY | M1_SEE_INVIS | M1_POIS | M1_FIRE_RES |
+	  M1_STALK | M1_PRINCE, M2_DEMON | M2_HOSTILE | M2_PNAME | M2_NASTY },
 	{ "Asmodeus", S_DEMON, 105, 12, -7, 90, 20,
 	  (G_HELL | G_NOCORPSE | G_NOGEN | G_UNIQ),
 	  { { AT_CLAW, AD_PHYS, 4, 4 }, { AT_MAGC, AD_COLD, 6, 6 },
 	    NO_ATTK, NO_ATTK, NO_ATTK }, 50, 500, PL_NSIZ, MS_JEER,
-	  M1_NOPOLY | M1_FLY | M1_HUMANOID | M1_POIS |
+	  M1_NOPOLY | M1_FLY | M1_SEE_INVIS | M1_HUMANOID | M1_POIS |
 	  M1_FIRE_RES | M1_COLD_RES | M1_STALK | M1_PRINCE,
 	  M2_DEMON | M2_HOSTILE | M2_PNAME | M2_STRONG | M2_NASTY },
 	{ "Demogorgon", S_DEMON, 106, 15, -8, 95, -20,
@@ -1374,14 +1376,15 @@ struct permonst _mons2[] = {
 	  { { AT_MAGC, AD_SPEL, 8, 6 }, { AT_STNG, AD_DRLI, 1, 4 },
 	    { AT_CLAW, AD_DISE, 1, 6 }, { AT_CLAW, AD_DISE, 1, 6 },
 	    NO_ATTK }, 50, 500, PL_NSIZ, MS_GROWL, M1_NOPOLY | M1_FLY |
-	  M1_NOHANDS | M1_POIS | M1_FIRE_RES | M1_STALK |
+	  M1_SEE_INVIS | M1_NOHANDS | M1_POIS | M1_FIRE_RES | M1_STALK |
 	  M1_PRINCE, M2_DEMON | M2_HOSTILE | M2_PNAME | M2_NASTY },
 #endif
 #ifdef MAIL
 	{ "mail daemon", S_DEMON, 56, 24, 10, 127, 0, (G_NOGEN | G_NOCORPSE),
 	  { NO_ATTK, NO_ATTK, NO_ATTK, NO_ATTK, NO_ATTK }, 30, 300, 0,
-	  MS_SILENT, M1_NOPOLY | M1_FLY | M1_SWIM | M1_HUMANOID | M1_POIS |
-	  M1_FIRE_RES | M1_COLD_RES | M1_SLEE_RES | M1_STALK, M2_PEACEFUL },
+	  MS_SILENT, M1_NOPOLY | M1_FLY | M1_SWIM | M1_SEE_INVIS |
+	  M1_HUMANOID | M1_POIS | M1_FIRE_RES | M1_COLD_RES | M1_SLEE_RES |
+	  M1_STALK, M2_PEACEFUL },
 /* Neither rain nor sleet nor gloom of night shall stay this courier... */
 #endif
 	{ "djinni", S_DEMON, 7, 12, 4, 30, 0, (G_NOGEN | G_NOCORPSE),
